import java.util.LinkedList;

public class Measures {

// RECURSIVE VERSIONS

	// returns a left comb of height h
	static Tree leftCombRec(int h) {
		if (h == 0) return null;
		return new Tree(leftCombRec(h-1), null);
	}
	
	// returns a perfect binary tree of height h
	static Tree perfectTreeRec(int h) {
		if (h == 0) return null;
		return new Tree(perfectTreeRec(h-1), perfectTreeRec(h-1));
	}
	
	// returns the size of t
	static int sizeRec(Tree t) {
		if (t == null) return 0;
		return 1 + sizeRec(t.left) + sizeRec(t.right);
	}
	
	// returns the canopy of t
	static int canopyRec(Tree t) {
		if (t == null) return 0;
		if (t.left == null && t.right == null) return 1;
		return canopyRec(t.left) + canopyRec(t.right);
	}
	
	// returns the height of t
	static int heightRec(Tree t) {
		if (t == null) return 0;
		return 1 + Math.max(heightRec(t.left), heightRec(t.right));
	}

// ITERATIVE VERSIONS

	// returns a left comb of height h
	static Tree leftCombIt(int h) {
		Tree t = null;
		for (int i = 0; i < h; i++)
			t = new Tree(t, null);
		return t;
	}

	// returns the size of t
	static int sizeIt(Tree t) {
		LinkedList<Tree> subtrees = new LinkedList<Tree>();
		subtrees.add(t);
		int size = 0;
		while(!subtrees.isEmpty()) {
			t = subtrees.poll();
			if (t != null) {
				size++;
				subtrees.add(t.left);
				subtrees.add(t.right);
			}
		}
		return size;
	}

	// returns the canopy of t
	static int canopyIt(Tree t) {
		LinkedList<Tree> subtrees = new LinkedList<Tree>();
		subtrees.add(t);
		int canopy = 0;
		while(!subtrees.isEmpty()) {
			t = subtrees.poll();
			if (t != null) {
				if (t.left == null && t.right == null)
					canopy++;
				subtrees.add(t.left);
				subtrees.add(t.right);
			}
		}
		return canopy;
	}

	// returns the height of t
	static int heightIt(Tree t) {
		LinkedList<Pair> subtrees = new LinkedList<Pair>();
		subtrees.add(new Pair(t,0));
		int height = 0;
		while(!subtrees.isEmpty()) {
			Pair p = subtrees.poll();
			t = p.t;
			int d = p.d;
			if (p.t != null) {
				height = Math.max(height, d+1);
				subtrees.add(new Pair(t.left, d+1));
				subtrees.add(new Pair(t.right, d+1));
			}
		}
		return height;		
	}
	
	public static void main(String[] args) {
		//leftCombRec(20000);
	}
}
